# -*- coding: utf-8 -*-
#
#  lib.gb_config.py
#
#  Copyright 2010 Miguel Ángel Molina <sslapp@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>
#
#   Este programa es software libre: usted puede redistribuirlo y/o modificarlo
#   bajo los términos de la Licencia Pública General GNU publicada
#   por la Fundación para el Software Libre, ya sea la versión 3
#   de la Licencia, o (a su elección) cualquier versión posterior.
#
#   Este programa se distribuye con la esperanza de que sea útil, pero
#   SIN GARANTÍA ALGUNA; ni siquiera la garantía implícita
#   MERCANTIL o de APTITUD PARA UN PROPÓSITO DETERMINADO.
#   Consulte los detalles de la Licencia Pública General GNU para obtener
#   una información más detallada.
#
#   Debería haber recibido una copia de la Licencia Pública General GNU
#   junto a este programa.
#   En caso contrario, consulte <http://www.gnu.org/licenses/>.

import base64
import ConfigParser
import getpass
import os
import stat

class ConfigHandler:
  """Class to handle configuration options.

  """

  def __init__(self, main_frame=None):
    """ConfigHandler constructor."""

    self.main_frame = main_frame
    # Gibareto path
    self.gibareto_path = os.path.expanduser('~/.gibareto/')
    self.config_path = os.path.join(self.gibareto_path, 'gibareto.conf')

    # Check if config directory exists and create it if not
    if not os.path.isdir(self.gibareto_path):
      try:
        os.makedirs(self.gibareto_path, 0700)
      except OSError as error:
        if self.main_frame != None:
          self.main_frame.statusBar1.SetStatusText(error.strerror + ' : ' + error.filename)
        else:
          print error.strerror, error.filename
        return -1

    self.config = ConfigParser.SafeConfigParser()
    # Read config file
    self.config.read(self.config_path)
    # Check if configured
    if not self.config.has_section('Gibareto'):
      self.set_defaults()

  def set_defaults(self):
    """Set configuration defaults."""

    self.config.add_section('Gibareto')
    self.set_def_account()
    self.set_mail_host()
    self.set_backup_path()
    self.set_dates()
    self.set_from_last()
    self.set_text_font()
    self.set_text_color()
    self.save_config()

  def set_def_account(self, account=''):
    """Set default account."""

    self.config.set('Gibareto', 'def_account', account)

  def get_def_account(self):
    """Get default account."""

    return {'def_account':self.config.get('Gibareto', 'def_account')}

  def set_mail_host(self, p_mail_host='imap.gmail.com', p_mail_host_port='993'):
    """Set mail host url and mail host port."""

    self.config.set('Gibareto', 'mail_host', p_mail_host)
    self.config.set('Gibareto', 'mail_host_port', p_mail_host_port)

  def get_mail_host(self):
    """Get mail host url and mail host port."""

    return {'mail_host':self.config.get('Gibareto', 'mail_host'),
            'mail_host_port':self.config.get('Gibareto', 'mail_host_port')}

  def set_backup_path(self, p_backup_path=os.path.expanduser('~/gb_backups')):
    """Set base backup path."""

    # Check if backup path exists and create it if not
    if (not os.path.isdir(p_backup_path)):
      try:
        os.makedirs(p_backup_path, 0700)
      except OSError as error:
        if self.main_frame != None:
          self.main_frame.statusBar1.SetStatusText(error.strerror + ' : ' + error.filename)
        else:
          print error.strerror, error.filename
        return -1
    self.config.set('Gibareto', 'backup_path', p_backup_path)

  def get_backup_path(self):
    """Get base backup path."""

    return {'backup_path':self.config.get('Gibareto', 'backup_path')}

  def edit_global_data(self, p_mail_host='imap.gmail.com', p_mail_host_port='993',
                        p_backup_path=os.path.expanduser('~/gb_backups')):
    """Edit mail host, mail host port and backup path."""

    mail_host = raw_input(_("Mail host") + " [" + p_mail_host + "]: ")
    if mail_host == '':
      mail_host = p_mail_host

    while True:
      mail_host_port = raw_input(_("Mail host port") + " [" + p_mail_host_port + "]: ")
      if mail_host_port == '':
        mail_host_port = p_mail_host_port
      if mail_host_port.isdigit():
        break
      else:
        print _("Please, enter a valid port number")

    backup_path = raw_input(_("Enter backup base path") + " [" + p_backup_path + "]: ")
    if backup_path == '':
      backup_path = p_backup_path

    # Save data into config
    self.set_mail_host(mail_host, mail_host_port)
    self.set_backup_path(backup_path)

  def set_dates(self, p_date_from='', p_date_to=''):
    """Set from and to dates."""

    self.config.set('Gibareto', 'date_from', p_date_from)
    self.config.set('Gibareto', 'date_to', p_date_to)

  def get_dates(self):
    """Get from and to dates."""

    return {'date_from':self.config.get('Gibareto', 'date_from'),
            'date_to':self.config.get('Gibareto', 'date_to')}

  def edit_dates(self, p_date_from='', p_date_to=''):
    """Edit from and to dates."""

    while True:
      date_from = raw_input(_("Enter date from [ddmmyyyy]: "))
      if date_from == '' or date_from.isdigit() and len(date_from) == 8:
        break
    while True:
      date_to = raw_input(_("Enter date to [ddmmyyyy]: "))
      if date_to == '' or date_to.isdigit() and len(date_to) == 8:
        break

    # Save data into config
    self.set_dates(date_from, date_to)

  def set_from_last(self, p_from_last=True):
    """Set From Last flag."""

    if p_from_last:
      from_last = 'True'
    else:
      from_last = ''
    self.config.set('Gibareto', 'from_last', from_last)

  def get_from_last(self):
    """Get From Last flag."""

    return {'from_last':bool(self.config.get('Gibareto', 'from_last'))}

  def edit_from_last(self):
    """Edit From Last flag."""

    enable_from_last = raw_input(_("Do you want to enable from_last [Y/n]? ")).upper()
    if enable_from_last == _("N"):
      from_last = False
    else:
      from_last = True

    #Save data into config
    self.set_from_last(from_last)

  def set_text_font(self, font=''):
    """Set Font for Log window."""

    self.config.set('Gibareto', 'text_font', font)

  def get_text_font(self):
    """Get Font for Log window."""

    return {'text_font':self.config.get('Gibareto', 'text_font')}

  def set_text_color(self, color=''):
    """Set Font Color for Log window."""

    self.config.set('Gibareto', 'text_color', color)

  def get_text_color(self):
    """Get Font Color for Log window."""

    return {'text_color':self.config.get('Gibareto', 'text_color')}

  def set_account(self, p_user_name, p_password):
    """Set user name and password."""

    user_name_name = p_user_name.split('@')[0]
    # Create appropiate config section
    if not self.config.has_section(user_name_name):
      self.config.add_section(user_name_name)
    self.config.set(user_name_name, 'user_name', p_user_name)
    self.config.set(user_name_name, 'password', p_password)

  def get_account(self, p_account):
    """Get user name and password."""

    return {'user_name':self.config.get(p_account, 'user_name'),
            'password':self.config.get(p_account, 'password')}

  def get_accounts(self):
    """Get accounts list."""

    accounts = self.config.sections()[:]
    # Remove global section
    accounts.remove('Gibareto')
    accounts.sort()
    return accounts

  def edit_account(self, p_user_name='yourname@gmail.com', p_password=''):
    """Edit user name and password."""

    result = ''
    print _("Enter account data:")

    user_name = raw_input(_("Enter your account user name") + " [" + p_user_name + "]: ")
    if user_name == '':
      user_name = p_user_name
    user_name_name = user_name.split('@')[0]

    save_pass = 'N'
    password = p_password
    save_pass = raw_input(_("Save encrypted password to config file? [y/N] ")).upper()
    if save_pass == _("Y"):
      password = base64.b64encode(getpass.getpass(_("Enter your account password: ")))
    else:
      password = ''
    default_account = raw_input(_("Make this the default account? [y/N] ")).upper()
    if default_account == _('Y'):
      self.set_def_account(user_name_name)
      # If default account is set return it
      result = user_name_name

    # Save data into config
    self.set_account(user_name, password)
    return result

  def delete_account(self, account):
    """Delete account."""

    self.config.remove_section(account)
    result = ''
    if self.get_def_account()['def_account'] == account:
      self.set_def_account()
      result = 'default_deleted'
    self.save_config()
    return result

  def save_config(self):
    """Save config to file."""
    cf = open(self.config_path, 'wb')
    self.config.write(cf)
    cf.close()
    os.chmod(self.config_path, stat.S_IRUSR | stat.S_IWUSR)
