# -*- coding: utf-8 -*-
#
#  lib.gb_imap.py
#
#  IMAP classes and functions interface
#
#  Copyright 2010 Miguel Ángel Molina <sslapp@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>
#
#   Este programa es software libre: usted puede redistribuirlo y/o modificarlo
#   bajo los términos de la Licencia Pública General GNU publicada
#   por la Fundación para el Software Libre, ya sea la versión 3
#   de la Licencia, o (a su elección) cualquier versión posterior.
#
#   Este programa se distribuye con la esperanza de que sea útil, pero
#   SIN GARANTÍA ALGUNA; ni siquiera la garantía implícita
#   MERCANTIL o de APTITUD PARA UN PROPÓSITO DETERMINADO.
#   Consulte los detalles de la Licencia Pública General GNU para obtener
#   una información más detallada.
#
#   Debería haber recibido una copia de la Licencia Pública General GNU
#   junto a este programa.
#   En caso contrario, consulte <http://www.gnu.org/licenses/>.

import base64
import email
import getpass
import imaplib
import re

# Mail Server Class
class MailServer:
  """class to represent the Mail Server.

      Attributes:
      mail_server   ->  mail server URL
      user_name     ->  account user name
      password      ->  account password
      capabilities  ->  mail server capabilities
      mailboxes     ->  list of mailboxes as MailBox objects

      Methods:
      connect               ->  establish a connection to the mail server
      login                 ->  login into mail server
      logout                ->  logout from mail server
      get_mailboxes         ->  get mailboxes names

  """

  def __init__(self, mail_server_addr, mail_server_port, user_name, password):
    """MailServer class constructor.

        Params:
        mail_server_addr  ->  mail server URL
        mail_server_port  ->  mail server port
        user_name         ->  account user name
        password          ->  account password

    """

    self.mail_server_addr = mail_server_addr
    self.mail_server_port = int(mail_server_port)
    self.user_name = user_name
    self.password = base64.b64decode(password)

  def connect(self):
    """Connect to mail server."""

    # Connect to mail server
    self.mail_server = imaplib.IMAP4_SSL(self.mail_server_addr, self.mail_server_port)

    # Get the mail server capabilities
    self.capabilities = self.mail_server.capabilities

  def login(self):
    """Login into the mail server."""

    self.connect()
    self.mail_server.login(self.user_name, self.password)

  def logout(self):
    """Logout from the mail server."""

    if self.mail_server.state == 'SELECTED':
      self.mail_server.close()
    self.mail_server.logout()

  def get_mailboxes(self):
    """Get the mailboxes."""

    self.mailboxes = {}
    dummy, mailboxes = self.mail_server.list()
    for mailbox in mailboxes:
      name = mailbox.split(' "/" ')[1][1:-1]
      d_index = name.lower()
      d_index = d_index.replace('[gmail]','')
      d_index = d_index.replace('/','')
      sub_name = name.split('/')[-1]
      if (d_index != ""):
        self.mailboxes[d_index] = MailBox(self.mail_server, name, sub_name)
    # try to identify the All label
    n_max = 0
    n_key = ''
    for k, v in self.mailboxes.iteritems():
      if v.n_msgs > n_max:
        n_key = k
        n_max = v.n_msgs
    if n_key != '':
      # Modify the key, so 'All' mailbox will be scanned last
      self.mailboxes['zz' + n_key] = self.mailboxes[n_key]
      del(self.mailboxes[n_key])

  def print_mailboxes(self):
    """Print mailboxes list."""

    mbk =self.mailboxes.keys()
    mbk.sort()
    for mb in mbk:
      print self.mailboxes[mb]

class MailBox:
  """Class to represent a Mailbox."""

  def __init__(self, mail_server, name, sub_name):
    self.mail_server = mail_server
    self.name = name
    self.sub_name = sub_name
    self.n_msgs = self.get_mailbox_n_msgs()

  def __repr__(self):
    return '{0:.<25s}\t{1:.>5d}'.format(self.name,self.n_msgs)

  def get_mailbox_n_msgs(self):
    """Get mailbox number of messages."""

    dummy, data = self.mail_server.select(self.name,True)
    return int(data[0])

  def get_message_list(self, date_from='', date_to=''):
    """Get message list within dates."""

    dummy, data = self.mail_server.select(self.name,True)
    if date_from == '' and date_to == '':
      dummy, data = self.mail_server.search(None,'ALL')
    elif date_to == '':
      dummy, data = self.mail_server.search(None,'(SINCE ' + date_from + ')')
    elif date_from == '':
      dummy, data = self.mail_server.search(None,'(OR ON ' + date_to + ' BEFORE '+ date_to + ')')
    else:
      dummy, data = self.mail_server.search(None,'(SINCE ' + date_from + ' BEFORE '+ date_to + ')')

    message_list = data[0].split()
    return message_list

  def get_message_id(self, msg_id):
    """Get the Message-ID header field of a message.

        Returns a Message-ID
    """

    id = ''
    dummy, data = self.mail_server.fetch(msg_id, '(BODY.PEEK[HEADER.FIELDS (Message-ID)])')
    for msg in data:
      if not isinstance(msg, tuple):
        continue
      match = re.search("<(.+)>", msg[1])
      if match:
        id = match.group(1)
    return base64.urlsafe_b64encode(id)

  def get_message(self, msg_seq, id):
    """Get complete message.

        Params:
        msg_seq ->  Message sequence number
        id      ->  Message-ID

        Returns an instance of MailMessage
    """

    re_size = re.compile(r'RFC822.SIZE (?P<size>[0-9]+).*')
    dummy, data = self.mail_server.fetch(msg_seq, '(BODY.PEEK[] INTERNALDATE RFC822.SIZE)')
    body = data[0][1]
    rest = data[0][0]
    getsize = re_size.search(rest)
    if getsize:
      size = long(getsize.group('size'))
    internal_date = imaplib.Internaldate2tuple(rest)
    e_mail = email.message_from_string(body)
    return MailMessage(id, size, internal_date, e_mail)

class MailMessage:
  """Class to represent a mail message."""

  def __init__(self, id, size, date, e_mail):
    self.id = id
    self.size = size
    self.date = date
    self.e_mail = e_mail
